/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
* Version: NPL 1.1/GPL 2.0/LGPL 2.1
*
* The contents of this file are subject to the Netscape Public License
* Version 1.1 (the "License"); you may not use this file except in
* compliance with the License. You may obtain a copy of the License at
* http://www.mozilla.org/NPL/
*
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
* for the specific language governing rights and limitations under the
* License.
*
* The Original Code is mozilla.org code.
*
* The Initial Developer of the Original Code is
* Netscape Communications Corporation.
* Portions created by the Initial Developer are Copyright (C) 2002
* the Initial Developer. All Rights Reserved.
*
* Contributor(s):
*    David Haas <haasd@cae.wisc.edu>
*
*
* Alternatively, the contents of this file may be used under the terms of
* either the GNU General Public License Version 2 or later (the "GPL"), or
* the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
* in which case the provisions of the GPL or the LGPL are applicable instead
* of those above. If you wish to allow use of your version of this file only
* under the terms of either the GPL or the LGPL, and not to allow others to
* use your version of this file under the terms of the NPL, indicate your
* decision by deleting the provisions above and replace them with the notice
* and other provisions required by the GPL or the LGPL. If you do not delete
* the provisions above, a recipient may use your version of this file under
* the terms of any one of the NPL, the GPL or the LGPL.
*
* ***** END LICENSE BLOCK ***** */

#import "BookmarkItem.h"

//Update Frequency Flags
#define kBookmarkNeverUpdate 0
#define kBookmarkMinutelyUpdate 1
#define kBookmarkHourlyUpdate 2
#define kBookmarkDailyUpdate 3
#define kBookmarkWeeklyUpdate 4

//Status Flags
#define kBookmarkOKStatus 0
#define kBookmarkUpdatedStatus 1
#define kBookmarkBrokenLinkStatus 2
#define kBookmarkMovedLinkStatus 3
#define kBookmarkServerErrorStatus 4
#define kBookmarkSpacerStatus 9

//Notification type flag
enum {
  kNotificationNothingSpecial = 0,
  kNotificationBeepOnUpdate = 1 << 0,
  kNotificationIconOnUpdate = 1 << 1
};

@interface Bookmark : BookmarkItem  //DBME
{
  NSString* mURL; //DBMu
  NSDate* mLastVisit; //DBMv
  NSNumber* mStatus;
  NSNumber* mNumberOfVisits;
  NSNumber* mUpdateTimeIncrement; //DBMn
  NSNumber* mUpdateTimeUnits; //DBMt
  NSNumber* mUpdateNotification;
  NSTimer* mUpdateTimer;
  NSLock* mLock;
}

-(NSString *) url; //protected by lock
-(NSDate *) lastVisit; //protected by lock
-(unsigned) numberOfVisits;
-(unsigned)  status;
-(BOOL) isSeparator;
-(BOOL) isUpdated;  //DHIu
-(BOOL) isSick;     //DHIs
-(BOOL) iconOnUpdate;
-(BOOL) beepOnUpdate;

-(unsigned) updateTimeIncrement;
-(unsigned) updateTimeUnits;
-(NSTimer *) updateTimer; 

-(void) setUrl:(NSString *)aURL; //protected by lock
-(void) setLastVisit:(NSDate *)aLastVisit; //protected by lock
-(void) setStatus:(unsigned)aStatus;
-(void) setUpdateTimeIncrement:(unsigned)aNumber;
-(void) setUpdateTimeUnits:(unsigned)aUnit;
-(void) setUpdateTimer:(NSTimer *)aTimer; 
-(void) setIsSeparator:(BOOL)aSeparatorFlag;
-(void) setNumberOfVisits:(unsigned)aNumber;

-(void) setBeepOnUpdate:(BOOL)aFlag;
-(void) setIconOnUpdate:(BOOL)aFlag;


// functions aiding in checking for updates
-(NSURL *)urlAsURL;
-(void) checkForUpdate:(NSTimer *)aTimer;

@end

